@@ -98,10 +98,10 @@ static void menu_applet_constructed(GObject *obj)
 	    true);
 	self->button        = NULL;
 	GSettings *settings = vala_panel_applet_get_settings(VALA_PANEL_APPLET(self));
+	g_settings_bind(settings, MODEL_FILE, self, MODEL_FILE, G_SETTINGS_BIND_GET);
 	g_settings_bind(settings, IS_SYSTEM, self, IS_SYSTEM, G_SETTINGS_BIND_GET);
 	g_settings_bind(settings, IS_BAR, self, IS_BAR, G_SETTINGS_BIND_GET);
 	g_settings_bind(settings, IS_INTERNAL, self, IS_INTERNAL, G_SETTINGS_BIND_GET);
-	g_settings_bind(settings, MODEL_FILE, self, MODEL_FILE, G_SETTINGS_BIND_GET);
 	g_settings_bind(settings, ICON, self, ICON, G_SETTINGS_BIND_GET);
 	g_settings_bind(settings, CAPTION, self, CAPTION, G_SETTINGS_BIND_GET);
 	GtkSettings *gtksettings = gtk_widget_get_settings(GTK_WIDGET(self));
@@ -109,7 +109,8 @@ static void menu_applet_constructed(GObject *obj)
 	gtk_widget_show(GTK_WIDGET(self));
 }
 
-static void panel_gravity_changed(ValaPanelToplevel *panel, GParamSpec *param, GtkMenuBar *menu)
+static void panel_gravity_changed(ValaPanelToplevel *panel, G_GNUC_UNUSED GParamSpec *param,
+                                  GtkMenuBar *menu)
 {
 	GtkOrientation orient;
 	GtkPackDirection pack;
@@ -138,13 +139,12 @@ static GtkContainer *create_menubar(MenuApplet *self)
 
 static void on_menubutton_toggled(GtkToggleButton *b, void *data)
 {
-	MenuApplet *self = VALA_PANEL_MENU_APPLET(data);
+	MenuApplet *self       = VALA_PANEL_MENU_APPLET(data);
+	ValaPanelToplevel *top = vala_panel_applet_get_toplevel(VALA_PANEL_APPLET(self));
+	GdkGravity menug, widget;
+	vala_panel_toplevel_get_menu_anchors(top, &menug, &widget);
 	if (gtk_toggle_button_get_active(b) && !gtk_widget_get_visible(GTK_WIDGET(self->int_menu)))
-		gtk_menu_popup_at_widget(self->int_menu,
-		                         GTK_WIDGET(self),
-		                         GDK_GRAVITY_NORTH,
-		                         GDK_GRAVITY_NORTH,
-		                         NULL);
+		gtk_menu_popup_at_widget(self->int_menu, GTK_WIDGET(self), widget, menug, NULL);
 	else
 		gtk_menu_popdown(self->int_menu);
 }
@@ -173,25 +173,24 @@ static void menubutton_create_image(MenuApplet *self, GtkToggleButton *menubutto
 
 static GtkContainer *create_menubutton(MenuApplet *self)
 {
-	GtkImage *img               = NULL;
-	GtkToggleButton *menubutton = gtk_toggle_button_new();
+	GtkWidget *menubutton = gtk_toggle_button_new();
 	if (!self->menu)
-		return menubutton;
-	self->int_menu           = gtk_menu_new_from_model(self->menu);
+		return GTK_CONTAINER(menubutton);
+	self->int_menu           = GTK_MENU(gtk_menu_new_from_model(G_MENU_MODEL(self->menu)));
 	g_autoptr(GList) ch_list = gtk_container_get_children(GTK_CONTAINER(self->int_menu));
-	apply_menu_properties(ch_list, self->menu);
+	apply_menu_properties(ch_list, G_MENU_MODEL(self->menu));
 	g_clear_pointer(&ch_list, g_list_free);
 	gtk_menu_attach_to_widget(self->int_menu, menubutton, NULL);
 	g_signal_connect(menubutton, "toggled", G_CALLBACK(on_menubutton_toggled), self);
 	g_signal_connect(self->int_menu, "hide", G_CALLBACK(on_menu_hide), menubutton);
-	menubutton_create_image(self, menubutton);
+	menubutton_create_image(self, GTK_TOGGLE_BUTTON(menubutton));
 	gtk_widget_show(menubutton);
-	return menubutton;
+	return GTK_CONTAINER(menubutton);
 }
 
 static GtkContainer *menumodel_widget_create(MenuApplet *self)
 {
-	self->menu        = create_menumodel(self);
+	self->menu        = G_MENU(create_menumodel(self));
 	GtkContainer *ret = NULL;
 	if (!self->menu)
 	{
@@ -206,28 +205,24 @@ static GtkContainer *menumodel_widget_create(MenuApplet *self)
 
 static void menumodel_widget_destroy(MenuApplet *self)
 {
-	vala_panel_applet_set_background_widget(self, self);
+	ValaPanelToplevel *top = vala_panel_applet_get_toplevel(VALA_PANEL_APPLET(self));
+	vala_panel_applet_set_background_widget(VALA_PANEL_APPLET(self), GTK_WIDGET(self));
 	if (self->monitor_update_idle)
 		g_source_remove(self->monitor_update_idle);
 	if (self->show_system_menu_idle)
 		g_source_remove(self->show_system_menu_idle);
-	if (self->button)
-	{
-		g_signal_handlers_disconnect_by_data(vala_panel_applet_get_toplevel(self),
-		                                     self->button);
-		g_signal_handlers_disconnect_by_data(self->button, self);
-	}
 	if (GTK_IS_WIDGET(self->int_menu))
 	{
 		if (self->button)
-		{
 			g_signal_handlers_disconnect_by_data(self->int_menu, self->button);
-			gtk_menu_detach(self->int_menu);
-		}
-		if (GTK_IS_WIDGET(self->int_menu))
-			gtk_widget_destroy0(self->int_menu);
+		gtk_widget_destroy(GTK_WIDGET(self->int_menu));
+	}
+	if (GTK_IS_WIDGET(self->button))
+	{
+		g_signal_handlers_disconnect_by_data(top, self->button);
+		g_signal_handlers_disconnect_by_data(self->button, self);
+		gtk_widget_destroy(GTK_WIDGET(self->button));
 	}
-	gtk_widget_destroy0(self->button);
 	if (G_IS_OBJECT(self->menu))
 		g_clear_object(&self->menu);
 	if (self->app_monitor)
@@ -263,7 +258,7 @@ static void menumodel_widget_rebuild(MenuApplet *self)
 	menumodel_widget_destroy(self);
 	self->button = menumodel_widget_create(self);
 	if (GTK_IS_WIDGET(self->button))
-		gtk_container_add(self, self->button);
+		gtk_container_add(GTK_CONTAINER(self), GTK_WIDGET(self->button));
 }
 
 static void load_internal_menus(GMenu *menu, MenuInternalEnum enum_id)
@@ -429,7 +424,7 @@ static void menu_applet_set_property(GObject *object, uint prop_id, const GValue
 		break;
 	case PROP_FILENAME:
 		g_free0(self->icon);
-		self->caption = g_value_dup_string(value);
+		self->filename = g_value_dup_string(value);
 		if (!self->intern)
 			menumodel_widget_rebuild(self);
 		break;
@@ -470,15 +465,14 @@ static GtkWidget *menu_applet_get_settings_ui(ValaPanelApplet *self)
 
 static int show_menu_int(void *data)
 {
-	MenuApplet *self = VALA_PANEL_MENU_APPLET(data);
+	MenuApplet *self       = VALA_PANEL_MENU_APPLET(data);
+	ValaPanelToplevel *top = vala_panel_applet_get_toplevel(VALA_PANEL_APPLET(self));
+	GdkGravity menug, widget;
+	vala_panel_toplevel_get_menu_anchors(top, &menug, &widget);
 	if (g_source_is_destroyed(g_main_current_source()))
 		return false;
 	if (GTK_IS_MENU(self->int_menu))
-		gtk_menu_popup_at_widget(self->int_menu,
-		                         GTK_WIDGET(self),
-		                         GDK_GRAVITY_NORTH,
-		                         GDK_GRAVITY_NORTH,
-		                         NULL);
+		gtk_menu_popup_at_widget(self->int_menu, GTK_WIDGET(self), widget, menug, NULL);
 	else
 		gtk_menu_shell_select_first(GTK_MENU_SHELL(self->button), false);
 	self->show_system_menu_idle = 0;
@@ -546,7 +540,7 @@ static void menu_applet_class_init(MenuAppletClass *klass)
 	g_object_class_install_properties(G_OBJECT_CLASS(klass), NUM_PROPERTIES, menu_applet_props);
 }
 
-static void menu_applet_class_finalize(MenuAppletClass *klass)
+static void menu_applet_class_finalize(G_GNUC_UNUSED MenuAppletClass *klass)
 {
 }
 
@@ -560,7 +554,6 @@ void g_io_menumodel_load(GTypeModule *module)
 
 	menu_applet_register_type(module);
 
-	g_type_module_use(module);
 	g_io_extension_point_implement(VALA_PANEL_APPLET_EXTENSION_POINT,
 	                               menu_applet_get_type(),
 	                               "org.valapanel.menumodel",
