@@ -25,11 +25,11 @@ static ValaPanelAppletManager *manager;
 
 enum
 {
-	PROP_DUMMY,
-	PROP_TOPLEVEL_ID,
-	PROP_LAST
+	LAYOUT_DUMMY,
+	LAYOUT_TOPLEVEL_ID,
+	LAYOUT_LAST
 };
-static GParamSpec *vp_layout_properties[PROP_LAST];
+static GParamSpec *vp_layout_properties[LAYOUT_LAST];
 
 struct _ValaPanelLayout
 {
@@ -277,7 +277,9 @@ G_GNUC_INTERNAL void vp_layout_remove_applet(ValaPanelLayout *self, ValaPanelApp
 	g_autofree char *uuid        = g_strdup(vala_panel_applet_get_uuid(applet));
 	uint pos                     = vp_layout_get_applet_position(self, applet);
 	ValaPanelAppletPackType type = vp_layout_get_applet_pack_type(applet);
-	g_clear_pointer(&applet, gtk_widget_destroy);
+	ValaPanelUnitSettings *unit  = vp_core_settings_get_by_uuid(core_settings, uuid);
+	g_signal_handlers_disconnect_by_data(unit->common, applet);
+	gtk_widget_destroy(GTK_WIDGET(applet));
 	vp_layout_applets_reposition_after(self, type, pos, GTK_PACK_START);
 	vp_core_settings_remove_unit_settings_full(core_settings, uuid, true);
 	vp_layout_update_applet_positions(self);
@@ -466,7 +468,7 @@ static void vp_layout_set_property(GObject *object, guint property_id, const GVa
 	ValaPanelLayout *self = VALA_PANEL_LAYOUT(object);
 	switch (property_id)
 	{
-	case PROP_TOPLEVEL_ID:
+	case LAYOUT_TOPLEVEL_ID:
 		g_free0(self->toplevel_id);
 		self->toplevel_id = g_value_dup_string(value);
 		break;
@@ -481,7 +483,7 @@ static void vp_layout_get_property(GObject *object, guint property_id, GValue *v
 	ValaPanelLayout *self = VALA_PANEL_LAYOUT(object);
 	switch (property_id)
 	{
-	case PROP_TOPLEVEL_ID:
+	case LAYOUT_TOPLEVEL_ID:
 		g_value_set_string(value, self->toplevel_id);
 		break;
 	default:
@@ -546,12 +548,12 @@ static void vp_layout_class_init(ValaPanelLayoutClass *klass)
 	G_OBJECT_CLASS(klass)->set_property = vp_layout_set_property;
 	G_OBJECT_CLASS(klass)->get_property = vp_layout_get_property;
 	G_OBJECT_CLASS(klass)->dispose      = vp_layout_destroy;
-	vp_layout_properties[PROP_TOPLEVEL_ID] =
+	vp_layout_properties[LAYOUT_TOPLEVEL_ID] =
 	    g_param_spec_string(VALA_PANEL_TOPLEVEL_ID,
 	                        VALA_PANEL_TOPLEVEL_ID,
 	                        VALA_PANEL_TOPLEVEL_ID,
 	                        NULL,
 	                        (GParamFlags)(G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE |
 	                                      G_PARAM_CONSTRUCT_ONLY));
-	g_object_class_install_properties(G_OBJECT_CLASS(klass), PROP_LAST, vp_layout_properties);
+	g_object_class_install_properties(G_OBJECT_CLASS(klass), LAYOUT_LAST, vp_layout_properties);
 }
