@@ -6,10 +6,6 @@
 G_BEGIN_DECLS
 
 #include "applet-manager.h"
-#include "applet-plugin.h"
-
-G_GNUC_INTERNAL ValaPanelApplet *vala_panel_applet_plugin_get_applet_widget(
-    ValaPanelAppletPlugin *self, ValaPanelToplevel *top, GSettings *settings, const char *uuid);
 
 /* From toplevel.h */
 G_GNUC_INTERNAL ValaPanelPlatform *vp_toplevel_get_current_platform(void);
@@ -21,6 +17,7 @@ G_GNUC_INTERNAL void vp_toplevel_destroy_pref_dialog(ValaPanelToplevel *self);
 G_GNUC_INTERNAL bool vp_toplevel_release_event_helper(GtkWidget *_sender, GdkEventButton *b,
                                                       gpointer obj);
 
+/* From platform.h */
 G_GNUC_INTERNAL ValaPanelAppletManager *vp_platform_get_manager(ValaPanelPlatform *self);
 
 G_END_DECLS
