@@ -18,7 +18,6 @@
 
 #include "toplevel-config.h"
 #include "applet-manager.h"
-#include "applet-plugin.h"
 #include "definitions.h"
 #include "panel-layout.h"
 #include "toplevel.h"
@@ -58,8 +57,8 @@ static GQuark applet_quark;
 	VALA_PANEL_APPLET(g_object_get_qdata((GObject *)row, applet_quark))
 #define config_row_set_applet(row, applet) g_object_set_qdata((GObject *)row, applet_quark, applet)
 
-#define config_row_get_info_data(row)                                                              \
-	((AppletInfoData *)(g_object_get_qdata((GObject *)row, applet_quark)))
+#define config_row_get_info(row)                                                                   \
+	((ValaPanelAppletInfo *)(g_object_get_qdata((GObject *)row, applet_quark)))
 #define config_row_set_info(row, applet) g_object_set_qdata((GObject *)row, applet_quark, applet)
 
 #define NEW_APPLET_NAME "new-applet"
@@ -109,6 +108,9 @@ static void vp_toplevel_config_init(ValaPanelToplevelConfig *self)
 static void vp_toplevel_config_finalize(GObject *obj)
 {
 	ValaPanelToplevelConfig *self = VP_TOPLEVEL_CONFIG(obj);
+	gtk_widget_insert_action_group(GTK_WIDGET(self), "conf", NULL);
+	gtk_widget_insert_action_group(GTK_WIDGET(self), "win", NULL);
+	gtk_widget_insert_action_group(GTK_WIDGET(self), "app", NULL);
 	g_clear_object(&self->monitors_box);
 	g_clear_object(&self->spin_iconsize);
 	g_clear_object(&self->spin_height);
@@ -393,9 +395,9 @@ static int plugin_list_sort(GtkListBoxRow *before, GtkListBoxRow *after, void *u
 	ValaPanelLayout *layout      = VALA_PANEL_LAYOUT(user_data);
 	ValaPanelApplet *before_info = before ? config_row_get_applet(before) : NULL;
 	ValaPanelApplet *after_info  = after ? config_row_get_applet(after) : NULL;
-	uint bpos                    = vp_layout_get_applet_position(layout, before_info);
-	uint apos                    = vp_layout_get_applet_position(layout, after_info);
-	uint bi                      = vp_layout_get_applet_pack_type(before_info);
+	uint bpos = before_info ? vp_layout_get_applet_position(layout, before_info) : 0;
+	uint apos = after_info ? vp_layout_get_applet_position(layout, after_info) : UINT_MAX;
+	uint bi   = before_info ? vp_layout_get_applet_pack_type(before_info) : PACK_START;
 
 	if (before_info != NULL && after_info != NULL &&
 	    vp_layout_get_applet_pack_type(before_info) !=
@@ -426,7 +428,8 @@ static int plugin_list_sort(GtkListBoxRow *before, GtkListBoxRow *after, void *u
 /**
  * Provide headers in the list to separate the visual positions
  */
-static void plugin_list_headers(GtkListBoxRow *before, GtkListBoxRow *after, void *user_data)
+static void plugin_list_headers(GtkListBoxRow *before, GtkListBoxRow *after,
+                                G_GNUC_UNUSED void *user_data)
 {
 	ValaPanelApplet *before_info = before ? config_row_get_applet(before) : NULL;
 	ValaPanelApplet *after_info  = after ? config_row_get_applet(after) : NULL;
@@ -554,25 +557,25 @@ static void on_about_plugin(G_GNUC_UNUSED GtkButton *btn, void *data)
 	vala_panel_applet_info_show_about_dialog(info);
 }
 
-static int listbox_new_name_sort(GtkListBoxRow *before, GtkListBoxRow *after, void *user_data)
+static int listbox_new_name_sort(GtkListBoxRow *before, GtkListBoxRow *after,
+                                 G_GNUC_UNUSED void *user_data)
 {
-	AppletInfoData *before_info = before ? config_row_get_info_data(before) : NULL;
-	AppletInfoData *after_info  = after ? config_row_get_info_data(after) : NULL;
+	ValaPanelAppletInfo *before_info = before ? config_row_get_info(before) : NULL;
+	ValaPanelAppletInfo *after_info  = after ? config_row_get_info(after) : NULL;
 	if (before_info && after_info)
-		return g_strcmp0(vala_panel_applet_info_get_name(before_info->info),
-		                 vala_panel_applet_info_get_name(after_info->info));
+		return g_strcmp0(vala_panel_applet_info_get_name(before_info),
+		                 vala_panel_applet_info_get_name(after_info));
 	else if (before_info)
 		return -1;
 	else
 		return 1;
 }
 
-static void listbox_new_applet_row_activated(GtkListBox *box, GtkListBoxRow *row,
+static void listbox_new_applet_row_activated(G_GNUC_UNUSED GtkListBox *box, GtkListBoxRow *row,
                                              gpointer user_data)
 {
 	ValaPanelToplevelConfig *self = VP_TOPLEVEL_CONFIG(user_data);
-	const char *type =
-	    vala_panel_applet_info_get_module_name(config_row_get_info_data(row)->info);
+	const char *type    = vala_panel_applet_info_get_module_name(config_row_get_info(row));
 	GtkListBoxRow *prev = gtk_list_box_get_selected_row(self->plugin_list);
 	plugin_list_add_applet(type, self, prev);
 	gtk_list_box_invalidate_filter(self->listbox_new_applet);
@@ -580,11 +583,9 @@ static void listbox_new_applet_row_activated(GtkListBox *box, GtkListBoxRow *row
 
 static int listbox_new_filter(GtkListBoxRow *row, G_GNUC_UNUSED void *data)
 {
-	AppletInfoData *d = config_row_get_info_data(row);
-	bool ret          = false;
-	if ((d->count < 1) || !vala_panel_applet_info_is_exclusive(d->info))
-		ret = true;
-	return ret;
+	ValaPanelAppletInfo *d = config_row_get_info(row);
+	return vp_applet_manager_is_applet_available(vp_layout_get_manager(),
+	                                             vala_panel_applet_info_get_module_name(d));
 }
 
 static void destroy(GtkWidget *applet, G_GNUC_UNUSED void *data)
@@ -603,7 +604,7 @@ static void available_plugins_reload(ValaPanelToplevelConfig *self)
 	{
 		AppletInfoData *d  = (AppletInfoData *)l->data;
 		GtkListBoxRow *row = create_info_representation(d->info);
-		config_row_set_info(row, d);
+		config_row_set_info(row, d->info);
 		gtk_container_add(GTK_CONTAINER(self->listbox_new_applet), GTK_WIDGET(row));
 	}
 	g_list_free(all_types);
@@ -635,25 +636,25 @@ static void on_add_plugin(G_GNUC_UNUSED GtkButton *btn, ValaPanelToplevelConfig
 	gtk_stack_set_visible_child_name(self->applet_info_stack, NEW_APPLET_NAME);
 }
 
-static inline GtkListBoxRow *get_row_from_btn(GtkButton *self)
+static inline GtkWidget *get_row_from_btn(GtkButton *self)
 {
-	return GTK_LIST_BOX_ROW(gtk_widget_get_ancestor(GTK_WIDGET(self), GTK_TYPE_LIST_BOX_ROW));
+	return gtk_widget_get_ancestor(GTK_WIDGET(self), GTK_TYPE_LIST_BOX_ROW);
 }
 
 static void on_remove_plugin(GtkButton *btn, void *user_data)
 {
 	ValaPanelToplevelConfig *self = VP_TOPLEVEL_CONFIG(user_data);
-	GtkListBoxRow *row            = get_row_from_btn(btn);
+	GtkWidget *row                = get_row_from_btn(btn);
 	if (!row)
 		return;
 	ValaPanelApplet *pl = config_row_get_applet(row);
 	GtkWidget *w =
 	    gtk_stack_get_child_by_name(self->applet_info_stack, vala_panel_applet_get_uuid(pl));
-	int index              = gtk_list_box_row_get_index(row);
+	int index              = gtk_list_box_row_get_index(GTK_LIST_BOX_ROW(row));
 	GtkListBoxRow *sel_row = gtk_list_box_get_selected_row(self->plugin_list);
 	int sel_index          = sel_row ? gtk_list_box_row_get_index(sel_row) : -1;
-	gtk_widget_destroy0(row);
-	gtk_widget_destroy0(w);
+	gtk_container_remove(GTK_CONTAINER(self->plugin_list), row);
+	g_clear_object(&w);
 	if (index == sel_index)
 		gtk_list_box_select_row(self->plugin_list,
 		                        gtk_list_box_get_row_at_index(self->plugin_list, index));
