@@ -19,14 +19,19 @@
 #ifndef TOPLEVELCONFIG_H
 #define TOPLEVELCONFIG_H
 
+#include "toplevel.h"
 #include <glib-object.h>
 #include <glib.h>
 #include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
-G_DECLARE_FINAL_TYPE(ValaPanelToplevelConfig, vala_panel_toplevel_config, VALA_PANEL,
-                     TOPLEVEL_CONFIG, GtkDialog)
+G_DECLARE_FINAL_TYPE(ValaPanelToplevelConfig, vp_toplevel_config, VP, TOPLEVEL_CONFIG, GtkDialog)
+
+G_GNUC_INTERNAL void vp_toplevel_config_select_applet(ValaPanelToplevelConfig *self,
+                                                      const char *uuid);
+G_GNUC_INTERNAL void vp_toplevel_config_select_page(ValaPanelToplevelConfig *self,
+                                                    const char *page);
 
 G_END_DECLS
 
